@@ -167,8 +167,16 @@ def codegen_main():
                             help='Use "pragma once" as the inclusion guard')
     arg_parser.add_argument('--annotate', nargs=3, action='append', metavar='WHAT KEY VALUE',
                             help='Add annotation (may be used several times)')
-    arg_parser.add_argument('--glib-min-version', metavar='VERSION',
+    arg_parser.add_argument('--glib-min-required', metavar='VERSION',
                             help='Minimum version of GLib to be supported by the outputted code (default: 2.30)')
+    arg_parser.add_argument('--glib-max-allowed', metavar='VERSION',
+                            help='Maximum version of GLib to be used by the outputted code (default: current GLib version)')
+    arg_parser.add_argument('--symbol-decorator',
+                            help='Macro used to decorate a symbol in the outputted header, possibly to export symbols')
+    arg_parser.add_argument('--symbol-decorator-header',
+                            help='Additional header required for decorator specified by --symbol-decorator')
+    arg_parser.add_argument('--symbol-decorator-define',
+                            help='Additional define required for decorator specified by --symbol-decorator')
 
     group = arg_parser.add_mutually_exclusive_group()
     group.add_argument('--generate-c-code', metavar='OUTFILES',
@@ -235,30 +243,52 @@ def codegen_main():
         c_file = args.output
         header_name = os.path.splitext(os.path.basename(c_file))[0] + '.h'
 
-    # Check the minimum GLib version. The minimum --glib-min-version is 2.30,
+    # Check the minimum GLib version. The minimum --glib-min-required is 2.30,
     # because that’s when gdbus-codegen was introduced. Support 1, 2 or 3
     # component versions, but ignore the micro component if it’s present.
-    if args.glib_min_version:
+    if args.glib_min_required:
         try:
-            parts = args.glib_min_version.split('.', 3)
-            glib_min_version = (int(parts[0]),
+            parts = args.glib_min_required.split('.', 3)
+            glib_min_required = (int(parts[0]),
+                                 int(parts[1] if len(parts) > 1 else 0))
+            # Ignore micro component, but still validate it:
+            _ = int(parts[2] if len(parts) > 2 else 0)
+        except (ValueError, IndexError):
+            print_error('Unrecognized --glib-min-required string ‘{}’'.format(
+                args.glib_min_required))
+
+        if glib_min_required < (2, 30):
+            print_error('Invalid --glib-min-required string ‘{}’: minimum '
+                        'version is 2.30'.format(args.glib_min_required))
+    else:
+        glib_min_required = (2, 30)
+
+    # And the maximum GLib version.
+    if args.glib_max_allowed:
+        try:
+            parts = args.glib_max_allowed.split('.', 3)
+            glib_max_allowed = (int(parts[0]),
                                 int(parts[1] if len(parts) > 1 else 0))
             # Ignore micro component, but still validate it:
             _ = int(parts[2] if len(parts) > 2 else 0)
         except (ValueError, IndexError):
-            print_error('Unrecognized --glib-min-version string ‘{}’'.format(
-                args.glib_min_version))
-
-        if glib_min_version[0] < 2 or \
-           (glib_min_version[0] == 2 and glib_min_version[1] < 30):
-            print_error('Invalid --glib-min-version string ‘{}’: minimum '
-                        'version is 2.30'.format(args.glib_min_version))
+            print_error('Unrecognized --glib-max-allowed string ‘{}’'.format(
+                args.glib_max_allowed))
     else:
-        glib_min_version = (2, 30)
+        glib_max_allowed = (config.MAJOR_VERSION, config.MINOR_VERSION)
 
-    glib_min_version_is_2_64 = (glib_min_version[0] > 2 or
-                                (glib_min_version[0] == 2 and
-                                 glib_min_version[1] >= 64))
+    # Only allow --symbol-decorator-define and --symbol-decorator-header if --symbol-decorator is used
+    if args.symbol_decorator is None:
+        if args.symbol_decorator_header or args.symbol_decorator_define:
+            print_error('--symbol-decorator-define and --symbol-decorator-header must be used with --symbol-decorator')
+
+    # Round --glib-max-allowed up to the next stable release.
+    glib_max_allowed = \
+        (glib_max_allowed[0], glib_max_allowed[1] + (glib_max_allowed[1] % 2))
+
+    if glib_max_allowed < glib_min_required:
+        print_error('Invalid versions: --glib-min-required ({}) must be '
+                    'less than or equal to --glib-max-allowed ({})'.format(glib_min_required, glib_max_allowed))
 
     all_ifaces = []
     input_files_basenames = []
@@ -266,7 +296,7 @@ def codegen_main():
         with open(fname, 'rb') as f:
             xml_data = f.read()
         parsed_ifaces = parser.parse_dbus_xml(xml_data,
-                                              h_type_implies_unix_fd=glib_min_version_is_2_64)
+                                              h_type_implies_unix_fd=(glib_min_required >= (2, 64)))
         all_ifaces.extend(parsed_ifaces)
         input_files_basenames.append(os.path.basename(fname))
 
@@ -290,7 +320,9 @@ def codegen_main():
                                               header_name,
                                               input_files_basenames,
                                               args.pragma_once,
-                                              glib_min_version,
+                                              glib_min_required,
+                                              args.symbol_decorator,
+                                              args.symbol_decorator_header,
                                               outfile)
             gen.generate()
 
@@ -302,7 +334,8 @@ def codegen_main():
                                         header_name,
                                         input_files_basenames,
                                         docbook_gen,
-                                        glib_min_version,
+                                        glib_min_required,
+                                        args.symbol_decorator_define,
                                         outfile)
             gen.generate()
 
@@ -313,7 +346,9 @@ def codegen_main():
                                                            header_name,
                                                            input_files_basenames,
                                                            args.pragma_once,
-                                                           glib_min_version,
+                                                           glib_min_required,
+                                                           args.symbol_decorator,
+                                                           args.symbol_decorator_header,
                                                            outfile)
             gen.generate()
 
@@ -323,7 +358,8 @@ def codegen_main():
                                                          args.c_namespace,
                                                          header_name,
                                                          input_files_basenames,
-                                                         glib_min_version,
+                                                         glib_min_required,
+                                                         args.symbol_decorator_define,
                                                          outfile)
             gen.generate()
 
