@@ -87,7 +87,7 @@ test_object_finalize (GObject *gobject)
 
   /* When the ref_count of an object is zero it is still
    * possible to notify the property, but it should do
-   * nothing and silenty quit (bug #705570)
+   * nothing and silently quit (bug #705570)
    */
   g_object_notify (gobject, "foo");
   g_object_notify_by_pspec (gobject, properties[PROP_BAR]);
@@ -223,54 +223,54 @@ typedef struct {
 static void
 on_notify (GObject           *gobject,
            GParamSpec        *pspec,
-           TestNotifyClosure *clos)
+           TestNotifyClosure *closure)
 {
-  g_assert (clos->pspec == pspec);
-  g_assert_cmpstr (clos->name, ==, pspec->name);
-  clos->fired = TRUE;
+  g_assert (closure->pspec == pspec);
+  g_assert_cmpstr (closure->name, ==, pspec->name);
+  closure->fired = TRUE;
 }
 
 static void
 properties_notify (void)
 {
   TestObject *obj = g_object_new (test_object_get_type (), NULL);
-  TestNotifyClosure clos;
+  TestNotifyClosure closure;
 
   g_assert (properties[PROP_FOO] != NULL);
   g_assert (properties[PROP_QUUX] != NULL);
-  g_signal_connect (obj, "notify", G_CALLBACK (on_notify), &clos);
+  g_signal_connect (obj, "notify", G_CALLBACK (on_notify), &closure);
 
-  clos.name = "foo";
-  clos.pspec = properties[PROP_FOO];
+  closure.name = "foo";
+  closure.pspec = properties[PROP_FOO];
 
-  clos.fired = FALSE;
+  closure.fired = FALSE;
   g_object_set (obj, "foo", 47, NULL);
-  g_assert (clos.fired);
+  g_assert (closure.fired);
 
-  clos.name = "baz";
-  clos.pspec = properties[PROP_BAZ];
+  closure.name = "baz";
+  closure.pspec = properties[PROP_BAZ];
 
-  clos.fired = FALSE;
+  closure.fired = FALSE;
   g_object_set (obj, "baz", "something new", NULL);
-  g_assert (clos.fired);
+  g_assert (closure.fired);
 
   /* baz lacks explicit notify, so we will see this twice */
-  clos.fired = FALSE;
+  closure.fired = FALSE;
   g_object_set (obj, "baz", "something new", NULL);
-  g_assert (clos.fired);
+  g_assert (closure.fired);
 
   /* quux on the other hand, ... */
-  clos.name = "quux";
-  clos.pspec = properties[PROP_QUUX];
+  closure.name = "quux";
+  closure.pspec = properties[PROP_QUUX];
 
-  clos.fired = FALSE;
+  closure.fired = FALSE;
   g_object_set (obj, "quux", "something new", NULL);
-  g_assert (clos.fired);
+  g_assert (closure.fired);
 
   /* no change; no notify */
-  clos.fired = FALSE;
+  closure.fired = FALSE;
   g_object_set (obj, "quux", "something new", NULL);
-  g_assert (!clos.fired);
+  g_assert (!closure.fired);
 
 
   g_object_unref (obj);
